package projekt.enigma;

import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import projekt.enigma.model.Fehler;

import java.io.IOException;

/**
 * JavaFX App
 * Startklasse fuer JavaFX Application, durch die Erweiterung um die Klasse Application.
 * Baut die Benutzeroberflaeche in ihren Grundstrukturen auf.
 */
public class App extends Application {

	private static String kenngruppe;
	private static GuiController gc;
	private static int debug;

	public static void main(String[] args) {
		debug = 1;
		launch(args);
	}

	public static int getDebug() {
		return debug;
	}

	@Override
	public void start(Stage primaryStage) throws IOException {

		primaryStage.setTitle("Kenngruppenabfrage");

		Label label1 = new Label("Bitte Kenngruppe eingeben!");
		label1.setTranslateX(55);
		Label l1 = new Label("");
		Label l2 = new Label("");
		TextField tfield = new TextField();
		Button button1 = new Button("Kenngruppe setzen!");
		VBox layout1 = new VBox();

		layout1.getChildren().addAll(label1, l1, tfield, l2, button1);
		Scene sceKenngruppe = new Scene(layout1, 234, 137);

		FXMLLoader guiLoader = new FXMLLoader(App.class.getResource("gui.fxml"));

		Scene sceApp = new Scene(guiLoader.load(), 962, 677);

		tfield.setOnKeyPressed(e -> {
			if (e.getCode() == KeyCode.ENTER) {
				if (enter(tfield.getText())) {
					gc = guiLoader.getController();
					gc.setKenngruppe(tfield.getText());
					primaryStage.setScene(sceApp);
				} else {
					tfield.setText("");
					new Fehler().showErrorDialog("error", "Kenngruppe falsch",
							"Regeln für die Kenngruppe: 4-8 Zeichen sowie nur Buchstaben von A-Z.");
				}
			}
		});

		button1.setOnAction(e -> {
			if (enter(tfield.getText())) {
				gc = guiLoader.getController();
				gc.setKenngruppe(tfield.getText());
				primaryStage.setScene(sceApp);
			} else {
				tfield.setText("");
				new Fehler().showErrorDialog("error", "Kenngruppe falsch",
						"Regeln für die Kenngruppe: 4-8 Zeichen sowie Buchstaben von A-Z.");
			}
		});

		primaryStage.setScene(sceKenngruppe);
		primaryStage.show();
	}

	private boolean enter(String kenngruppe) {

		String alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		boolean result = true;

		for (char c : kenngruppe.toUpperCase().toCharArray()) {
			if (!alphabet.contains(String.valueOf(c))) {
				result = false;
				break;
			}
		}
		if (kenngruppe.length() > 8 || kenngruppe.length() < 4) {
			result = false;
		}

		return result;
	}

//	/**
//	 * TODO Dokumentieren
//	 */
//	private static Scene scene;
//	/**
//	 * TODO Dokumentieren
//	 *
//	 * @param fxml
//	 * @throws IOException
//	 */
//	static void setRoot(String fxml) throws IOException {
//		scene.setRoot(loadFXML(fxml));
//
//	}
//
//	/**
//	 * TODO Dokumentieren
//	 *
//	 * @param fxml
//	 * @return
//	 * @throws IOException
//	 */
//	private static Parent loadFXML(String fxml) throws IOException {
//		FXMLLoader fxmlLoader = new FXMLLoader(App.class.getResource(fxml + ".fxml"));
//		return fxmlLoader.load();
//	}
//
//	/**
//	 * TODO Dokumentieren
//	 *
//	 * @param args
//	 */
//	public static void main(String[] args) {
//		Application.launch();
//	}
//
//	/**
//	 * TODO Dokumentieren
//	 *
//	 * @param stage
//	 * @throws IOException
//	 */
//	@Override
//	public void start(Stage stage) throws IOException {
//		scene = new Scene(loadFXML("kenngruppe"));
//		stage.setScene(scene);
//		stage.setResizable(true);
//		stage.show();
//	}

}
